@@ -23,12 +23,13 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#include <tstring.h>
-#include <tdebug.h>
-#include <bitset>
-#include <math.h>
-
 #include "mpcproperties.h"
+
+#include <array>
+#include <cmath>
+
+#include "tdebug.h"
+#include "tstring.h"
 #include "mpcfile.h"
 
 using namespace TagLib;
@@ -36,47 +37,26 @@ using namespace TagLib;
 class MPC::Properties::PropertiesPrivate
 {
 public:
-  PropertiesPrivate() :
-    version(0),
-    length(0),
-    bitrate(0),
-    sampleRate(0),
-    channels(0),
-    totalFrames(0),
-    sampleFrames(0),
-    trackGain(0),
-    trackPeak(0),
-    albumGain(0),
-    albumPeak(0) {}
-
-  int version;
-  int length;
-  int bitrate;
-  int sampleRate;
-  int channels;
-  uint totalFrames;
-  uint sampleFrames;
-  uint trackGain;
-  uint trackPeak;
-  uint albumGain;
-  uint albumPeak;
-  String flags;
+  int version { 0 };
+  int length { 0 };
+  int bitrate { 0 };
+  int sampleRate { 0 };
+  int channels { 0 };
+  unsigned int totalFrames { 0 };
+  unsigned long sampleFrames { 0 };
+  int trackGain { 0 };
+  int trackPeak { 0 };
+  int albumGain { 0 };
+  int albumPeak { 0 };
 };
 
 ////////////////////////////////////////////////////////////////////////////////
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
-MPC::Properties::Properties(const ByteVector &data, long streamLength, ReadStyle style) :
+MPC::Properties::Properties(File *file, offset_t streamLength, ReadStyle style) :
   AudioProperties(style),
-  d(new PropertiesPrivate())
-{
-  readSV7(data, streamLength);
-}
-
-MPC::Properties::Properties(File *file, long streamLength, ReadStyle style) :
-  AudioProperties(style),
-  d(new PropertiesPrivate())
+  d(std::make_unique<PropertiesPrivate>())
 {
   ByteVector magic = file->readBlock(4);
   if(magic == "MPCK") {
@@ -89,20 +69,7 @@ MPC::Properties::Properties(File *file, long streamLength, ReadStyle style) :
   }
 }
 
-MPC::Properties::~Properties()
-{
-  delete d;
-}
-
-int MPC::Properties::length() const
-{
-  return lengthInSeconds();
-}
-
-int MPC::Properties::lengthInSeconds() const
-{
-  return d->length / 1000;
-}
+MPC::Properties::~Properties() = default;
 
 int MPC::Properties::lengthInMilliseconds() const
 {
@@ -129,12 +96,12 @@ int MPC::Properties::mpcVersion() const
   return d->version;
 }
 
-TagLib::uint MPC::Properties::totalFrames() const
+unsigned int MPC::Properties::totalFrames() const
 {
   return d->totalFrames;
 }
 
-TagLib::uint MPC::Properties::sampleFrames() const
+unsigned long MPC::Properties::sampleFrames() const
 {
   return d->sampleFrames;
 }
@@ -163,55 +130,55 @@ int MPC::Properties::albumPeak() const
 // private members
 ////////////////////////////////////////////////////////////////////////////////
 
-unsigned long readSize(File *file, TagLib::uint &sizeLength, bool &eof)
-{
-  sizeLength = 0;
-  eof = false;
-
-  unsigned char tmp;
-  unsigned long size = 0;
-
-  do {
-    const ByteVector b = file->readBlock(1);
-    if(b.isEmpty()) {
-      eof = true;
-      break;
-    }
-
-    tmp = b[0];
-    size = (size << 7) | (tmp & 0x7F);
-    sizeLength++;
-  } while((tmp & 0x80));
-  return size;
-}
-
-unsigned long readSize(const ByteVector &data, TagLib::uint &pos)
-{
-  unsigned char tmp;
-  unsigned long size = 0;
-
-  do {
-    tmp = data[pos++];
-    size = (size << 7) | (tmp & 0x7F);
-  } while((tmp & 0x80) && (pos < data.size()));
-  return size;
-}
-
 namespace
 {
+  unsigned long readSize(File *file, unsigned int &sizeLength, bool &eof)
+  {
+    sizeLength = 0;
+    eof = false;
+
+    unsigned char tmp;
+    unsigned long size = 0;
+
+    do {
+      const ByteVector b = file->readBlock(1);
+      if(b.isEmpty()) {
+        eof = true;
+        break;
+      }
+
+      tmp = b[0];
+      size = (size << 7) | (tmp & 0x7F);
+      sizeLength++;
+    } while(tmp & 0x80);
+    return size;
+  }
+
+  unsigned long readSize(const ByteVector &data, unsigned int &pos)
+  {
+    unsigned char tmp;
+    unsigned long size = 0;
+
+    do {
+      tmp = data[pos++];
+      size = (size << 7) | (tmp & 0x7F);
+    } while((tmp & 0x80) && pos < data.size());
+    return size;
+  }
+
   // This array looks weird, but the same as original MusePack code found at:
   // https://www.musepack.net/index.php?pg=src
-  const unsigned short sftable [8] = { 44100, 48000, 37800, 32000, 0, 0, 0, 0 };
-}
+  constexpr std::array sftable { 44100, 48000, 37800, 32000, 0, 0, 0, 0 };
+}  // namespace
 
-void MPC::Properties::readSV8(File *file, long streamLength)
+void MPC::Properties::readSV8(File *file, offset_t streamLength)
 {
   bool readSH = false, readRG = false;
 
   while(!readSH && !readRG) {
     const ByteVector packetType = file->readBlock(2);
 
-    uint packetSizeLength;
+    unsigned int packetSizeLength;
     bool eof;
     const unsigned long packetSize = readSize(file, packetSizeLength, eof);
     if(eof) {
@@ -238,7 +205,7 @@ void MPC::Properties::readSV8(File *file, long streamLength)
 
       readSH = true;
 
-      TagLib::uint pos = 4;
+      unsigned int pos = 4;
       d->version = data[pos];
       pos += 1;
       d->sampleFrames = readSize(data, pos);
@@ -247,23 +214,22 @@ void MPC::Properties::readSV8(File *file, long streamLength)
         break;
       }
 
-      const ulong begSilence = readSize(data, pos);
+      const unsigned long begSilence = readSize(data, pos);
       if(pos > dataSize - 2) {
         debug("MPC::Properties::readSV8() - \"SH\" packet is corrupt.");
         break;
       }
 
-      const ushort flags = data.toUShort(pos, true);
-      pos += 2;
+      const unsigned short flags = data.toUShort(pos, true);
 
       d->sampleRate = sftable[(flags >> 13) & 0x07];
       d->channels   = ((flags >> 4) & 0x0F) + 1;
 
-      const uint frameCount = d->sampleFrames - begSilence;
-      if(frameCount > 0 && d->sampleRate > 0) {
-        const double length = frameCount * 1000.0 / d->sampleRate;
+      if(const auto frameCount = d->sampleFrames - begSilence;
+         frameCount > 0 && d->sampleRate > 0) {
+        const auto length = static_cast<double>(frameCount) * 1000.0 / d->sampleRate;
         d->length  = static_cast<int>(length + 0.5);
-        d->bitrate = static_cast<int>(streamLength * 8.0 / length + 0.5);
+        d->bitrate = static_cast<int>(static_cast<double>(streamLength) * 8.0 / length + 0.5);
       }
     }
     else if (packetType == "RG") {
@@ -277,8 +243,7 @@ void MPC::Properties::readSV8(File *file, long streamLength)
 
       readRG = true;
 
-      const int replayGainVersion = data[0];
-      if(replayGainVersion == 1) {
+      if(const int replayGainVersion = data[0]; replayGainVersion == 1) {
         d->trackGain = data.toShort(1, true);
         d->trackPeak = data.toShort(3, true);
         d->albumGain = data.toShort(5, true);
@@ -296,55 +261,57 @@ void MPC::Properties::readSV8(File *file, long streamLength)
   }
 }
 
-void MPC::Properties::readSV7(const ByteVector &data, long streamLength)
+void MPC::Properties::readSV7(const ByteVector &data, offset_t streamLength)
 {
   if(data.startsWith("MP+")) {
+    if(data.size() < 4)
+      return;
+
     d->version = data[3] & 15;
     if(d->version < 7)
       return;
 
     d->totalFrames = data.toUInt(4, false);
 
-    const uint flags = data.toUInt(8, false);
+    const unsigned int flags = data.toUInt(8, false);
     d->sampleRate = sftable[(flags >> 16) & 0x03];
     d->channels   = 2;
 
-    const uint gapless = data.toUInt(5, false);
+    const unsigned int gapless = data.toUInt(5, false);
 
     d->trackGain = data.toShort(14, false);
-    d->trackPeak = data.toShort(12, false);
+    d->trackPeak = data.toUShort(12, false);
     d->albumGain = data.toShort(18, false);
-    d->albumPeak = data.toShort(16, false);
+    d->albumPeak = data.toUShort(16, false);
 
     // convert gain info
     if(d->trackGain != 0) {
-      int tmp = (int)((64.82 - (short)d->trackGain / 100.) * 256. + .5);
+      auto tmp = static_cast<int>((64.82 - static_cast<short>(d->trackGain) / 100.) * 256. + .5);
       if(tmp >= (1 << 16) || tmp < 0) tmp = 0;
       d->trackGain = tmp;
     }
 
     if(d->albumGain != 0) {
-      int tmp = (int)((64.82 - d->albumGain / 100.) * 256. + .5);
+      auto tmp = static_cast<int>((64.82 - d->albumGain / 100.) * 256. + .5);
       if(tmp >= (1 << 16) || tmp < 0) tmp = 0;
       d->albumGain = tmp;
     }
 
     if (d->trackPeak != 0)
-      d->trackPeak = (int)(log10((double)d->trackPeak) * 20 * 256 + .5);
+      d->trackPeak = static_cast<int>(log10(static_cast<double>(d->trackPeak)) * 20 * 256 + .5);
 
     if (d->albumPeak != 0)
-      d->albumPeak = (int)(log10((double)d->albumPeak) * 20 * 256 + .5);
+      d->albumPeak = static_cast<int>(log10(static_cast<double>(d->albumPeak)) * 20 * 256 + .5);
 
-    bool trueGapless = (gapless >> 31) & 0x0001;
-    if(trueGapless) {
-      uint lastFrameSamples = (gapless >> 20) & 0x07FF;
+    if((gapless >> 31) & 0x0001) {
+      unsigned int lastFrameSamples = (gapless >> 20) & 0x07FF;
       d->sampleFrames = d->totalFrames * 1152 - lastFrameSamples;
     }
     else
       d->sampleFrames = d->totalFrames * 1152 - 576;
   }
   else {
-    const uint headerData = data.toUInt(0, false);
+    const unsigned int headerData = data.toUInt(0, false);
 
     d->bitrate    = (headerData >> 23) & 0x01ff;
     d->version    = (headerData >> 11) & 0x03ff;
@@ -360,10 +327,10 @@ void MPC::Properties::readSV7(const ByteVector &data, long streamLength)
   }
 
   if(d->sampleFrames > 0 && d->sampleRate > 0) {
-    const double length = d->sampleFrames * 1000.0 / d->sampleRate;
+    const auto length = static_cast<double>(d->sampleFrames) * 1000.0 / d->sampleRate;
     d->length = static_cast<int>(length + 0.5);
 
     if(d->bitrate == 0)
-      d->bitrate = static_cast<int>(streamLength * 8.0 / length + 0.5);
+      d->bitrate = static_cast<int>(static_cast<double>(streamLength) * 8.0 / length + 0.5);
   }
 }
