@@ -23,12 +23,11 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#include <tstring.h>
-#include <tdebug.h>
-
-#include <oggpageheader.h>
-
 #include "vorbisproperties.h"
+
+#include "tstring.h"
+#include "tdebug.h"
+#include "oggpageheader.h"
 #include "vorbisfile.h"
 
 using namespace TagLib;
@@ -36,24 +35,14 @@ using namespace TagLib;
 class Vorbis::Properties::PropertiesPrivate
 {
 public:
-  PropertiesPrivate() :
-    length(0),
-    bitrate(0),
-    sampleRate(0),
-    channels(0),
-    vorbisVersion(0),
-    bitrateMaximum(0),
-    bitrateNominal(0),
-    bitrateMinimum(0) {}
-
-  int length;
-  int bitrate;
-  int sampleRate;
-  int channels;
-  int vorbisVersion;
-  int bitrateMaximum;
-  int bitrateNominal;
-  int bitrateMinimum;
+  int length { 0 };
+  int bitrate { 0 };
+  int sampleRate { 0 };
+  int channels { 0 };
+  int vorbisVersion { 0 };
+  int bitrateMaximum { 0 };
+  int bitrateNominal { 0 };
+  int bitrateMinimum { 0 };
 };
 
 namespace TagLib {
@@ -61,8 +50,8 @@ namespace TagLib {
    * Vorbis headers can be found with one type ID byte and the string "vorbis" in
    * an Ogg stream.  0x01 indicates the setup header.
    */
-  static const char vorbisSetupHeaderID[] = { 0x01, 'v', 'o', 'r', 'b', 'i', 's', 0 };
-}
+  static constexpr char vorbisSetupHeaderID[] = { 0x01, 'v', 'o', 'r', 'b', 'i', 's', 0 };
+} // namespace TagLib
 
 ////////////////////////////////////////////////////////////////////////////////
 // public members
@@ -70,25 +59,12 @@ namespace TagLib {
 
 Vorbis::Properties::Properties(File *file, ReadStyle style) :
   AudioProperties(style),
-  d(new PropertiesPrivate())
+  d(std::make_unique<PropertiesPrivate>())
 {
   read(file);
 }
 
-Vorbis::Properties::~Properties()
-{
-  delete d;
-}
-
-int Vorbis::Properties::length() const
-{
-  return lengthInSeconds();
-}
-
-int Vorbis::Properties::lengthInSeconds() const
-{
-  return d->length / 1000;
-}
+Vorbis::Properties::~Properties() = default;
 
 int Vorbis::Properties::lengthInMilliseconds() const
 {
@@ -144,7 +120,7 @@ void Vorbis::Properties::read(File *file)
     return;
   }
 
-  uint pos = 0;
+  unsigned int pos = 0;
 
   if(data.mid(pos, 7) != vorbisSetupHeaderID) {
     debug("Vorbis::Properties::read() -- invalid Vorbis identification header");
@@ -156,7 +132,7 @@ void Vorbis::Properties::read(File *file)
   d->vorbisVersion = data.toUInt(pos, false);
   pos += 4;
 
-  d->channels = uchar(data[pos]);
+  d->channels = static_cast<unsigned char>(data[pos]);
   pos += 1;
 
   d->sampleRate = data.toUInt(pos, false);
@@ -169,7 +145,6 @@ void Vorbis::Properties::read(File *file)
   pos += 4;
 
   d->bitrateMinimum = data.toUInt(pos, false);
-  pos += 4;
 
   // Find the length of the file.  See http://wiki.xiph.org/VorbisStreamLength/
   // for my notes on the topic.
@@ -182,13 +157,16 @@ void Vorbis::Properties::read(File *file)
     const long long end   = last->absoluteGranularPosition();
 
     if(start >= 0 && end >= 0 && d->sampleRate > 0) {
-      const long long frameCount = end - start;
-
-      if(frameCount > 0) {
-        const double length = frameCount * 1000.0 / d->sampleRate;
-
+      if(const long long frameCount = end - start; frameCount > 0) {
+        const auto length = static_cast<double>(frameCount) * 1000.0 / d->sampleRate;
+        offset_t fileLengthWithoutOverhead = file->length();
+        // Ignore the three initial header packets, see "1.3.1. Decode Setup" in
+        // https://xiph.org/vorbis/doc/Vorbis_I_spec.html
+        for (unsigned int i = 0; i < 3; ++i) {
+          fileLengthWithoutOverhead -= file->packet(i).size();
+        }
         d->length  = static_cast<int>(length + 0.5);
-        d->bitrate = static_cast<int>(file->length() * 8.0 / length + 0.5);
+        d->bitrate = static_cast<int>(static_cast<double>(fileLengthWithoutOverhead) * 8.0 / length + 0.5);
       }
     }
     else {
